@@ -1,5 +1,6 @@
 /* getkey.c -  Get a key from the database
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002,
+ *               2003 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -38,7 +39,7 @@
 #define MAX_UID_CACHE_ENTRIES  200
 
 #if MAX_PK_CACHE_ENTRIES < 2
-  #error We need the cache for key creation
+#error We need the cache for key creation
 #endif
 
 
@@ -83,7 +84,7 @@ typedef struct keyid_list {
 #endif
 
 #if MAX_UID_CACHE_ENTRIES < 5
-    #error we really need the userid cache
+#error we really need the userid cache
 #endif
 typedef struct user_id_db {
     struct user_id_db *next;
@@ -118,7 +119,7 @@ print_stats()
 void
 cache_public_key( PKT_public_key *pk )
 {
-  #if MAX_PK_CACHE_ENTRIES
+#if MAX_PK_CACHE_ENTRIES
     pk_cache_entry_t ce;
     u32 keyid[2];
 
@@ -157,7 +158,7 @@ cache_public_key( PKT_public_key *pk )
     ce->pk = copy_public_key( NULL, pk );
     ce->keyid[0] = keyid[0];
     ce->keyid[1] = keyid[1];
-  #endif
+#endif
 }
 
 
@@ -265,7 +266,7 @@ cache_user_id( KBNODE keyblock )
 void
 getkey_disable_caches()
 {
-  #if MAX_PK_CACHE_ENTRIES
+#if MAX_PK_CACHE_ENTRIES
     {
 	pk_cache_entry_t ce, ce2;
 
@@ -278,7 +279,7 @@ getkey_disable_caches()
 	pk_cache_entries = 0;
 	pk_cache = NULL;
     }
-  #endif
+#endif
     /* fixme: disable user id cache ? */
 }
 
@@ -318,7 +319,7 @@ get_pubkey( PKT_public_key *pk, u32 *keyid )
     int internal = 0;
     int rc = 0;
 
-  #if MAX_PK_CACHE_ENTRIES
+#if MAX_PK_CACHE_ENTRIES
     {	/* Try to get it from the cache */
 	pk_cache_entry_t ce;
 	for( ce = pk_cache; ce; ce = ce->next ) {
@@ -329,7 +330,7 @@ get_pubkey( PKT_public_key *pk, u32 *keyid )
 	    }
 	}
     }
-  #endif
+#endif
     /* more init stuff */
     if( !pk ) {
 	pk = m_alloc_clear( sizeof *pk );
@@ -568,7 +569,7 @@ classify_user_id2( const char *name,
     memset (desc, 0, sizeof *desc);
     *force_exact = 0;
     /* skip leading spaces.  Fixme: what is with trailing spaces? */
-    for(s = name; *s && isspace(*s); s++ )
+    for(s = name; *s && spacep (s); s++ )
 	;
 
     switch (*s) {
@@ -649,7 +650,7 @@ classify_user_id2( const char *name,
             }
 
 	    /* check if a hexadecimal number is terminated by EOS or blank */
-	    if (hexlength && s[hexlength] && !isspace(s[hexlength])) {
+	    if (hexlength && s[hexlength] && !spacep(s+hexlength)) {
 		if (hexprefix)	    /* a "0x" prefix without correct */
 		    return 0;	    /* termination is an error */
 		else		    /* The first chars looked like */
@@ -733,6 +734,27 @@ classify_user_id (const char *name, KEYDB_SEARCH_DESC *desc)
     return classify_user_id2 (name, desc, &dummy);
 }
 
+static int
+skip_disabled(void *dummy,u32 *keyid)
+{
+  int rc,disabled=0;
+  PKT_public_key *pk=m_alloc_clear(sizeof(PKT_public_key));
+
+  rc = get_pubkey(pk, keyid);
+  if(rc)
+    {
+      log_error("error checking disabled status of %08lX: %s\n",
+ 		(ulong)keyid[1],g10_errstr(rc));
+      goto leave;
+    }
+ 
+  disabled=pk_is_disabled(pk);
+
+ leave:
+  free_public_key(pk);
+  return disabled;
+}
+
 /****************
  * Try to get the pubkey by the userid. This function looks for the
  * first pubkey certificate which has the given name in a user_id.
@@ -784,7 +806,7 @@ key_byname( GETKEY_CTX *retctx, STRLIST namelist,
  	   && ctx->items[n].mode!=KEYDB_SEARCH_MODE_FPR16
 	   && ctx->items[n].mode!=KEYDB_SEARCH_MODE_FPR20
 	   && ctx->items[n].mode!=KEYDB_SEARCH_MODE_FPR)
- 	  ctx->items[n].skipfnc=is_disabled;
+ 	  ctx->items[n].skipfnc=skip_disabled;
     }
 
     ctx->kr_handle = keydb_new (secmode);
@@ -1265,7 +1287,7 @@ fixup_uidnode ( KBNODE uidnode, KBNODE signode, u32 keycreated )
     uid->is_primary = 0;
     p = parse_sig_subpkt ( sig->hashed, SIGSUBPKT_PRIMARY_UID, NULL );
     if ( p && *p )
-        uid->is_primary = 1;
+        uid->is_primary = 2;
     /* We could also query this from the unhashed area if it is not in
      * the hased area and then later try to decide which is the better
      * there should be no security problem with this.
@@ -1310,7 +1332,6 @@ fixup_uidnode ( KBNODE uidnode, KBNODE signode, u32 keycreated )
     p = parse_sig_subpkt (sig->hashed, SIGSUBPKT_FEATURES, &n);
     if (p && n && (p[0] & 0x01))
         uid->mdc_feature = 1;
-            
 }
 
 static void
@@ -1574,8 +1595,6 @@ merge_selfsigs_main( KBNODE keyblock, int *r_revoked )
 	    else if ( k->pkt->pkttype == PKT_SIGNATURE && uidnode )
 	      {
 		PKT_signature *sig = k->pkt->pkt.signature;
-		u32 dummy;
-		int dum2;
 
 		if(sig->keyid[0] != kid[0] || sig->keyid[1]!=kid[1])
 		  {
@@ -1585,7 +1604,7 @@ merge_selfsigs_main( KBNODE keyblock, int *r_revoked )
 
 		    if(get_pubkey_fast(ultimate_pk,sig->keyid)==0
 		       && check_key_signature2(keyblock,k,ultimate_pk,
-					       NULL,&dummy,&dum2)==0
+					       NULL,NULL,NULL)==0
 		       && get_ownertrust(ultimate_pk)==TRUST_ULTIMATE)
 		      {
 			free_public_key(ultimate_pk);
@@ -1673,14 +1692,38 @@ merge_selfsigs_main( KBNODE keyblock, int *r_revoked )
         if ( k->pkt->pkttype == PKT_USER_ID &&
 	     !k->pkt->pkt.user_id->attrib_data) {
             PKT_user_id *uid = k->pkt->pkt.user_id;
-            if ( uid->is_primary && uid->created > uiddate ) {
-                uiddate = uid->created;
-                uidnode = k;
-            }
-            if ( !uid->is_primary && uid->created > uiddate2 ) {
-                uiddate2 = uid->created;
-                uidnode2 = k;
-            }
+            if (uid->is_primary)
+	      {
+		if(uid->created > uiddate)
+		  {
+		    uiddate = uid->created;
+		    uidnode = k;
+		  }
+		else if(uid->created==uiddate && uidnode)
+		  {
+		    /* The dates are equal, so we need to do a
+		       different (and arbitrary) comparison.  This
+		       should rarely, if ever, happen.  It's good to
+		       try and guarantee that two different GnuPG
+		       users with two different keyrings at least pick
+		       the same primary. */
+		    if(cmp_user_ids(uid,uidnode->pkt->pkt.user_id)>0)
+		      uidnode=k;
+		  }
+	      }
+	    else
+	      {
+		if(uid->created > uiddate2)
+		  {
+		    uiddate2 = uid->created;
+		    uidnode2 = k;
+		  }
+		else if(uid->created==uiddate2 && uidnode2)
+		  {
+		    if(cmp_user_ids(uid,uidnode2->pkt->pkt.user_id)>0)
+		      uidnode2=k;
+		  }
+	      }
         }
     }
     if ( uidnode ) {
@@ -1695,23 +1738,43 @@ merge_selfsigs_main( KBNODE keyblock, int *r_revoked )
         }
     }
     else if( uidnode2 ) {
-        /* none is flagged primary - use the latest user ID we have */
+        /* none is flagged primary - use the latest user ID we have,
+	   and disambiguate with the arbitrary packet comparison. */
         uidnode2->pkt->pkt.user_id->is_primary = 1;
     }
     else
       {
-	/* None of our uids were self-signed, so pick the first one to
-	   be the primary.  This is the best we can do here since
-	   there are no self sigs to date the uids. */
+	/* None of our uids were self-signed, so pick the one that
+	   sorts first to be the primary.  This is the best we can do
+	   here since there are no self sigs to date the uids. */
+
+	uidnode = NULL;
 
 	for(k=keyblock; k && k->pkt->pkttype != PKT_PUBLIC_SUBKEY;
 	    k = k->next )
 	  {
-	    if(k->pkt->pkttype==PKT_USER_ID &&
-	       !k->pkt->pkt.user_id->attrib_data)
+	    if(k->pkt->pkttype==PKT_USER_ID
+	       && !k->pkt->pkt.user_id->attrib_data)
 	      {
-		k->pkt->pkt.user_id->is_primary=1;
-		break;
+		if(!uidnode)
+		  {
+		    uidnode=k;
+		    uidnode->pkt->pkt.user_id->is_primary=1;
+		    continue;
+		  }
+		else
+		  {
+		    if(cmp_user_ids(k->pkt->pkt.user_id,
+				    uidnode->pkt->pkt.user_id)>0)
+		      {
+			uidnode->pkt->pkt.user_id->is_primary=0;
+			uidnode=k;
+			uidnode->pkt->pkt.user_id->is_primary=1;
+		      }
+		    else
+		      k->pkt->pkt.user_id->is_primary=0; /* just to be
+							    safe */
+		  }
 	      }
 	  }
       }
@@ -2071,14 +2134,14 @@ finish_lookup (GETKEY_CTX ctx)
     KBNODE k;
     KBNODE foundk = NULL;
     PKT_user_id *foundu = NULL;
-  #define USAGE_MASK  (PUBKEY_USAGE_SIG|PUBKEY_USAGE_ENC)
+#define USAGE_MASK  (PUBKEY_USAGE_SIG|PUBKEY_USAGE_ENC)
     unsigned int req_usage = ( ctx->req_usage & USAGE_MASK );
     /* Request the primary if we're certifying another key, and also
        if signing data while --pgp6 or --pgp7 is on since pgp 6 and 7
        do not understand signatures made by a signing subkey.  PGP 8
        does. */
     int req_prim = (ctx->req_usage & PUBKEY_USAGE_CERT) ||
-      ((opt.pgp6 || opt.pgp7) && (ctx->req_usage & PUBKEY_USAGE_SIG));
+      ((PGP6 || PGP7) && (ctx->req_usage & PUBKEY_USAGE_SIG));
     u32 latest_date;
     KBNODE latest_key;
     u32 curtime = make_timestamp ();
